@@ -12,7 +12,7 @@ use base qw(Net::DNS::Resolver::Base DynaLoader), OS_CONF;
 our $VERSION;
 
 BEGIN {
-	$VERSION = '1.27_04';
+	$VERSION = '1.30_01';
 	eval { __PACKAGE__->bootstrap($VERSION) };
 }
 
@@ -27,7 +27,6 @@ use constant IRRELEVENT => qw(igntc nameserver4 nameserver6 nameservers
 		persistent_tcp persistent_udp port retrans retry
 		srcaddr4 srcaddr6 srcport tcp_timeout udp_timeout usevc);
 
-use constant IP_PREF	 => UB_VERSION > 11100;
 use constant ADD_TA_AUTR => UB_CONTEXT->can('add_ta_autr');
 use constant SET_STUB	 => UB_CONTEXT->can('set_stub');
 use constant SET_TLS	 => UB_CONTEXT->can('set_tls');
@@ -39,10 +38,10 @@ Net::DNS::Resolver::Unbound - Net::DNS resolver based on libunbound
 
 =head1 SYNOPSIS
 
-    use Net::DNS;
-    use Net::DNS::Resolver::Unbound;
-    my $resolver = Net::DNS::Resolver::Unbound->new(...);
-    my $response = $resolver->send(...);
+	use Net::DNS;
+	use Net::DNS::Resolver::Unbound;
+	my $resolver = Net::DNS::Resolver::Unbound->new(...);
+	my $response = $resolver->send(...);
 
 =head1 DESCRIPTION
 
@@ -81,62 +80,128 @@ In particular, the queryID returned by Unbound is always zero.
 =back
 
 
+=head1 REPLACING Net::DNS::Resolver BASE CLASS
+
+Placing C<-register> in the L<Net::DNS::Resolver::Unbound> import list, will
+cause it to register itself with L<Net::DNS> as the resolver base class.
+
+	use Net::DNS;
+	use Net::DNS::Resolver::Unbound -register;
+	my $resolver = Net::DNS::Resolver->new(...);
+	my $response = $resolver->send(...);
+
+Note that "-register" is a global setting that applies to the entire
+program; it cannot be applied only for certain callers, removed, or
+limited by lexical scope.
+
+=cut
+
+sub import {
+	my ( $class, @argument ) = @_;
+	my $register = grep {/^-register$/i} @argument;
+	@Net::DNS::Resolver::ISA = $class if $register;
+}
+
+
 =head1 METHODS
 
 =head2 new
 
-    my $resolver = Net::DNS::Resolver::Unbound->new(
-	debug_level => 2,
-	defnames    => 1,
-	dnsrch,	    => 1,
-	domain	    => 'domain',
-	ndots	    => 1,
-	option	    => [ 'tls-cert-bundle', '/etc/ssl/cert.pem' ],
-	nameservers => [ ... ],
-	searchlist  => ['domain' ... ],
-	);
+	# Use the default configuration
+	my $resolver = Net::DNS::Resolver::Unbound->new();
+
+	# Set options in the constructor
+	my $resolver = Net::DNS::Resolver::Unbound->new(
+		debug_level => 2,
+		defnames    => 1,
+		dnsrch,	    => 1,
+		domain	    => 'domain',
+		nameservers => [ '2001:DB8::1', ... ],
+		ndots	    => 1,
+		searchlist  => ['domain' ... ]
+		);
 
 Returns a new Net::DNS::Resolver::Unbound resolver object.
+If no arguments are supplied, C<new()>
+returns an object having the default configuration.
+
+On Unix and Linux systems,
+the default values are read from the following files,
+in the order indicated:
+
+=over
+
+F</etc/resolv.conf>,
+F<$HOME/.resolv.conf>,
+F<./.resolv.conf>
+
+=back
+
+The following keywords are recognised in resolver configuration files:
+
+=over
+
+=item B<nameserver> address
+
+IP address of a name server that the resolver should query.
+
+=item B<domain> localdomain
+
+The domain suffix to be appended to a short non-absolute name.
+
+=item B<search> domain ...
+
+A space-separated list of domains in the desired search path.
+
+=item B<options> option:value ...
+
+A space-separated list of key:value items.
+
+=back
+
+Except for F</etc/resolv.conf>, files will only be read if owned by the
+effective userid running the program.
+
+Note that the domain and searchlist keywords are mutually exclusive.
+If both are present, the resulting behaviour is unspecified.
+If neither is present, the domain is determined from the local hostname.
 
 =cut
 
 sub new {
 	my ( $class, @args ) = @_;
-	my $self = $class->SUPER::new();
+	my $self = __PACKAGE__->SUPER::new();
 	$self->nameservers( $self->SUPER::nameservers );
 	delete $self->{$_} for IRRELEVENT;
 	$self->_finalise_config;				# default configuration
-	$self->{update} = {} if @args;				# force context rebuild
+	return $self unless @args;
 	while ( my $attr = shift @args ) {
 		my $value = shift @args;
 		$self->$attr( ref($value) ? @$value : $value );
 	}
+	$self->_finalise_config;				# force context rebuild
 	return $self;
 }
 
 
 =head2 nameservers
 
-    my $stub_resolver = Net::DNS::Resolver::Unbound->new(
-	nameserver => '127.0.0.53'
-	);
+	my $dnssec_resolver = Net::DNS::Resolver::Unbound->new(
+		nameservers => [],	# override /etc/resolv.conf
+		add_ta_file => '/var/lib/unbound/root.key'
+		);
 
-    my $fully_recursive = Net::DNS::Resolver::Unbound->new(
-	nameservers => [],		# override /etc/resolv.conf
-	add_ta_file => '/var/lib/unbound/root.key'
-	);
+	my $DoT_resolver = Net::DNS::Resolver::Unbound->new(
+		nameserver => '2606:4700:4700::1111@853#cloudflare-dns.com',
+		nameserver => '1.1.1.1@853#cloudflare-dns.com',
+		nameserver => '2001:4860:4860::8888@853#dns.google',
+		nameserver => '8.8.8.8@853#dns.google',
+		option	=> ['tls-cert-bundle' => '/etc/ssl/cert.pem'],
+		set_tls	=> 1
+		);
 
-    my $DoT_resolver = Net::DNS::Resolver->new(
-	nameserver => '2606:4700:4700::1111@853#cloudflare-dns.com',
-	nameserver => '2001:4860:4860::8888@853#dns.google',
-	nameserver => '8.8.8.8@853#dns.google',
-	nameserver => '9.9.9.9@853#dns.quad9.net',
-	option	   => [qw(tls-cert-bundle /etc/ssl/cert.pem)],
-	set_tls	   => 1
-	);
-
-    $resolver->nameservers( '::1', '127.0.0.1', ... );
-    @nameservers = $resolver->nameservers;
+	$resolver->nameservers( '::1', '127.0.0.1', ... );
+	@nameservers = $resolver->nameservers;
 
 By default, DNS queries are sent to the IP addresses listed in
 /etc/resolv.conf or similar platform-specific sources.
@@ -146,8 +211,8 @@ By default, DNS queries are sent to the IP addresses listed in
 sub nameservers {
 	my ( $self, @nameservers ) = @_;
 	if ( defined wantarray ) {
-		my $config   = $self->{config};
-		my $update   = $self->{update};
+		my $config   = $self->{ub_cfg};
+		my $update   = $self->{ub_upd};
 		my @setfwd   = ( $update->{set_fwd}, $config->{set_fwd}, [] );
 		my ($setfwd) = grep { defined $_ } @setfwd;
 		my @value    = map { ref($_) ? @$_ : $_ } $setfwd;
@@ -223,7 +288,7 @@ sub bgread {
 
 =head2 option
 
-    $resolver->option( 'tls-cert-bundle', '/etc/ssl/cert.pem' );
+	$resolver->option( 'tls-cert-bundle' => '/etc/ssl/cert.pem' );
 
 Set Unbound resolver (name,value) context option.
 
@@ -237,7 +302,7 @@ sub option {
 
 =head2 config
 
-    $resolver->config( 'Unbound.cfg' );
+	$resolver->config( 'Unbound.cfg' );
 
 This is a power-users interface that lets you specify all sorts of
 Unbound configuration options.
@@ -252,7 +317,7 @@ sub config {
 
 =head2 set_fwd
 
-    $resolver->set_fwd( 'IP address' );
+	$resolver->set_fwd( 'IP address' );
 
 Set IPv4 or IPv6 address to which DNS queries are to be directed.
 The destination machine is expected to run a recursive resolver.
@@ -270,8 +335,8 @@ sub set_fwd {
 
 =head2 set_tls
 
-    $resolver->set_tls( 0 );
-    $resolver->set_tls( 1 );
+	$resolver->set_tls( 0 );
+	$resolver->set_tls( 1 );
 
 Use DNS over TLS for queries to nameservers specified using set_fwd().
 
@@ -285,7 +350,7 @@ sub set_tls {
 
 =head2 set_stub
 
-    $resolver->set_stub( 'zone', '10.1.2.3', 0 );
+	$resolver->set_stub( 'zone', '10.1.2.3', 0 );
 
 Add a stub zone, with given address to send to. This is for custom root
 hints or pointing to a local authoritative DNS server. For DNS resolvers
@@ -301,7 +366,7 @@ sub set_stub {
 
 =head2 resolv_conf
 
-    $resolver->resolv_conf( 'filename' );
+	$resolver->resolv_conf( 'filename' );
 
 Extract nameserver list from resolv.conf(5) format configuration file.
 Any domain, searchlist, ndots or other settings are ignored.
@@ -319,7 +384,7 @@ sub resolv_conf {
 
 =head2 hosts
 
-    $resolver->hosts( 'filename' );
+	$resolver->hosts( 'filename' );
 
 Read list of hosts from the filename given, usually "/etc/hosts".
 These addresses are not flagged as DNSSEC secure when queried.
@@ -334,7 +399,7 @@ sub hosts {
 
 =head2 add_ta
 
-    $resolver->add_ta( 'trust anchor' );
+	$resolver->add_ta( 'trust anchor' );
 
 Add a trust anchor which is a string that holds a valid DNSKEY or DS RR
 in RFC1035 zonefile format.
@@ -350,7 +415,7 @@ sub add_ta {
 
 =head2 add_ta_file
 
-    $resolver->add_ta_file( 'filename' );
+	$resolver->add_ta_file( '/var/lib/unbound/root.key' );
 
 Pass the name of a file containing DS and DNSKEY records
 (as from dig or drill).
@@ -365,7 +430,7 @@ sub add_ta_file {
 
 =head2 add_ta_autr
 
-    $resolver->add_ta_autr( 'filename' );
+	$resolver->add_ta_autr( 'filename' );
 
 Add trust anchor to the given context that is tracked with RFC5011
 automated trust anchor maintenance.  The file is written when the
@@ -381,7 +446,7 @@ sub add_ta_autr {
 
 =head2 trusted_keys
 
-    $resolver->trusted_keys( 'filename' );
+	$resolver->trusted_keys( 'filename' );
 
 Pass the name of a BIND-style config file containing trusted-keys{}.
 
@@ -395,10 +460,10 @@ sub trusted_keys {
 
 =head2 debug_out
 
-    $resolver->debug_out( out );
+	$resolver->debug_out( out );
 
 Send debug output (and error output) to the specified stream.
-Pass a null argument to disable. Default is stderr.
+Pass a null argument to disable. The default is stderr.
 
 =cut
 
@@ -410,10 +475,11 @@ sub debug_out {
 
 =head2 debug_level
 
-    $resolver->debug_level(0);
+	$resolver->debug_level(0);
 
-Set verbosity of the debug output directed to stderr.  Level 0 is off,
-1 minimal, 2 detailed, 3 lots, and 4 lots more.
+Set verbosity of the debug output directed to stderr.
+The default level 0 is off, 1 minimal, 2 detailed, 3 lots,
+and 4 lots more.
 
 =cut
 
@@ -426,7 +492,7 @@ sub debug_level {
 
 =head2 async_thread
 
-    $resolver->async_thread(1);
+	$resolver->async_thread(1);
 
 Set the context behaviour for asynchronous actions.
 Enable a call to resolve_async() to create a thread to handle work in
@@ -443,8 +509,8 @@ sub async_thread {
 
 =head2 print, string
 
-    $resolver->print;
-    print $resolver->string;
+	$resolver->print;
+	print $resolver->string;
 
 Prints the resolver state on the standard output.
 
@@ -456,16 +522,16 @@ sub string {
 
 	my ($force)  = ( grep( { $self->{$_} } qw(force_v6 force_v4) ),	  'force_v4' );
 	my ($prefer) = ( grep( { $self->{$_} } qw(prefer_v6 prefer_v4) ), 'prefer_v4' );
-	my $image    = <<END;
+	my @search   = grep { defined($_) } @{$self->{searchlist}};
+	my @image    = <<END;
 ;; RESOLVER state:
-;; searchlist	@{$self->{searchlist}}
+;; searchlist	@search
 ;; defnames	$self->{defnames}	dnsrch	$self->{dnsrch}
 ;; ${prefer}	$self->{$prefer}	ndots	$self->{ndots}
 ;; ${force}	$self->{$force}	debug	$self->{debug}
 END
-	$self->{update} ||= {};					# force config rebuild
-	$self->_finalise_config;
-	my %config = %{$self->{config}};
+	$self->_finalise_config;				# force config rebuild
+	my %config = %{$self->{ub_cfg}};
 	my $optref = $config{set_option} || [];
 	my @option = @$optref;					# pre-sorted option list
 
@@ -483,13 +549,13 @@ END
 		if ( ref $value ) {
 			foreach my $arg (@$value) {
 				my @arg = map { ref($_) ? @$_ : $_ } $arg;
-				$image .= sprintf( $format, $name, join ' ', @arg );
+				push @image, sprintf( $format, $name, join ' ', @arg );
 			}
 		} else {
-			$image .= sprintf( $format, $name, $value );
+			push @image, sprintf( $format, $name, $value );
 		}
 	}
-	return $image;
+	return join '', @image;
 }
 
 
@@ -518,12 +584,12 @@ sub _config {
 	my $ctx	  = $self->{test_ctx};
 	$self->{test_ctx} = $ctx = Net::DNS::Resolver::Unbound::Context->new() unless $ctx;
 	$ctx->$name(@arg) if @arg;				# error check only
-	my $state = $self->{update}->{$name};
+	my $state = $self->{ub_upd}->{$name};
 	if ( defined $state ) {					# second and subsequent entries
-		$state = $self->{update}->{$name} = [$state] unless ref $state;
+		$state = $self->{ub_upd}->{$name} = [$state] unless ref $state;
 		push @$state, $entry;
 	} else {						# initial entry
-		$self->{update}->{$name} = ( scalar(@arg) > 1 ) ? [$entry] : $entry;
+		$self->{ub_upd}->{$name} = ( scalar(@arg) > 1 ) ? [$entry] : $entry;
 	}
 	return;
 }
@@ -536,11 +602,11 @@ sub _option {
 	my $ctx	    = $self->{test_ctx};
 	$self->{test_ctx} = $ctx = Net::DNS::Resolver::Unbound::Context->new() unless $ctx;
 	$ctx->set_option( $opt, @arg ) if defined $entry;	# error check only
-	my $updopt = $self->{update}->{set_option};
+	my $updopt = $self->{ub_upd}->{set_option};
 	my %option = map {$_} @$updopt;
 
 	unless (@arg) {
-		my $setopt = $self->{config}->{set_option};
+		my $setopt = $self->{ub_cfg}->{set_option};
 		my %config = map {$_} @$setopt, @$updopt;
 		my $value  = $config{$opt};
 		return ref($value) ? @$value : $value;
@@ -556,7 +622,7 @@ sub _option {
 
 	$option{$opt} = [] unless defined $entry;		# delete entire option
 	my @option = map { ( $_, $option{$_} ) } keys %option;
-	$self->{update}->{set_option} = \@option;
+	$self->{ub_upd}->{set_option} = \@option;
 	return;
 }
 
@@ -566,27 +632,29 @@ my %IP_conf = (
 	force_v6  => ['do-ip4:'	    => 'no'],
 	prefer_v4 => ['prefer-ip4:' => 'yes'],
 	prefer_v6 => ['prefer-ip6:' => 'yes'] );
-my @IPpref = map {@$_} values %IP_conf;
-delete @IP_conf{qw(prefer_v4 prefer_v6)} unless IP_PREF;
-my @IPconf = sort keys %IP_conf;
+my @IPconf = map {@$_} values %IP_conf;
+my @IPoptn = sort keys %IP_conf;
 
 sub _finalise_config {
 	my $self = shift;
 
-	my $update = delete $self->{update};
+	my @IPpref = grep { $self->{$_} } @IPoptn;		# from Net::DNS config
+	delete $self->{ub_upd}->{any} if @IPpref;		# force context rebuild
+
+	my $update = delete $self->{ub_upd};
 	return unless $update;
 	delete $self->{test_ctx};
 	delete $self->{ub_ctx};
 
-	my $config = $self->{config};
+	my $config = $self->{ub_cfg};
 	my $cfgopt = delete $config->{set_option};		# extract option lists
 	my $updopt = delete $update->{set_option};
 	my %config = ( %$config, %$update );			# merge config updates
 
 	my %option = map {$_} @$cfgopt, @$updopt;		# merge option updates
-	delete @option{@IPpref};				# expunge IP preference
-	foreach my $key ( grep { $self->$_ } @IPconf ) {	# insert IP preference
-		my ( $key, $value ) = @{$IP_conf{$key}};
+	delete @option{@IPconf};				# expunge IP preference
+	foreach ( @IPpref ) {					# insert IP preference
+		my ( $key, $value ) = @{$IP_conf{$_}};
 		$option{$key} = $value;
 		last;
 	}
@@ -607,7 +675,7 @@ sub _finalise_config {
 	}
 
 	$config{set_option} = \@option if @option;
-	$self->{config} = \%config;
+	$self->{ub_cfg} = \%config;
 	return;
 }
 
